package cn.com.qmth.examcloud.core.examwork.api.controller;

import static org.springframework.data.domain.ExampleMatcher.GenericPropertyMatchers.contains;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.JsonUtil;
import cn.com.qmth.examcloud.commons.web.helpers.page.PageInfo;
import cn.com.qmth.examcloud.commons.web.redis.RedisClient;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.OrgCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.OrgBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetOrgReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetOrgResp;
import cn.com.qmth.examcloud.core.examwork.api.controller.bean.CourseGroupDomain;
import cn.com.qmth.examcloud.core.examwork.api.controller.bean.ExamDomain;
import cn.com.qmth.examcloud.core.examwork.api.controller.bean.ExamOrgDomain;
import cn.com.qmth.examcloud.core.examwork.base.enums.ExamProperty;
import cn.com.qmth.examcloud.core.examwork.dao.CourseGroupRelationRepo;
import cn.com.qmth.examcloud.core.examwork.dao.CourseGroupRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamOrgPropertyRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamOrgRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamPropertyRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamStudentRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.CourseGroupEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.CourseGroupRelationEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamOrgEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamOrgPropertyEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamPropertyEntity;
import cn.com.qmth.examcloud.core.examwork.dao.enums.ExamType;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamInfo;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamOrgInfo;
import cn.com.qmth.examcloud.core.examwork.service.impl.ExamServiceImpl;
import cn.com.qmth.examcloud.core.examwork.service.impl.ExamStudentServiceImpl;
import io.swagger.annotations.ApiOperation;

/**
 * 重构
 *
 * @author WANGWEI
 * @date 2018年8月17日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Transactional
@RestController
@RequestMapping("${$rmp.ctr.examwork}/exam")
public class ExamController extends ControllerSupport {

	@Autowired
	ExamPropertyRepo examPropertyRepo;

	@Autowired
	RedisClient redisClient;

	@Autowired
	ExamRepo examRepo;

	@Autowired
	ExamServiceImpl examService;

	@Autowired
	ExamStudentRepo examStudentRepo;

	@Autowired
	ExamStudentServiceImpl examStudentService;

	@Autowired
	CourseGroupRepo courseGroupRepo;

	@Autowired
	ExamOrgRepo examOrgRepo;

	@Autowired
	CourseGroupRelationRepo courseGroupRelationRepo;

	@Autowired
	OrgCloudService orgCloudService;

	@Autowired
	ExamOrgPropertyRepo examOrgPropertyRepo;

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examCriteria
	 * @param curPage
	 * @param pageSize
	 * @return
	 */
	@ApiOperation(value = "查询所有考试批次", notes = "分页带查询")
	@GetMapping("all/{curPage}/{pageSize}")
	public Page<ExamEntity> getAllExam(@ModelAttribute ExamEntity examCriteria,
			@PathVariable Integer curPage, @PathVariable Integer pageSize) {
		User accessUser = getAccessUser();
		examCriteria.setRootOrgId(accessUser.getRootOrgId());

		ExampleMatcher exampleMatcher = ExampleMatcher.matching().withMatcher("name", contains());
		Example<ExamEntity> examExample = Example.of(examCriteria, exampleMatcher);
		return examRepo.findAll(examExample,
				new PageRequest(curPage, pageSize, new Sort(Direction.DESC, "updateTime")));
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examCriteria
	 * @return
	 */
	@ApiOperation(value = "查询所有考试批次(包含有效)", notes = "不分页带查询")
	@GetMapping("all")
	public List<ExamEntity> getEnableExam(@ModelAttribute ExamEntity examCriteria) {
		User accessUser = getAccessUser();
		examCriteria.setRootOrgId(accessUser.getRootOrgId());
		examCriteria.setEnable(true);
		ExampleMatcher exampleMatcher = ExampleMatcher.matching().withMatcher("name", contains());
		Example<ExamEntity> examExample = Example.of(examCriteria, exampleMatcher);
		return examRepo.findAll(examExample, new Sort(Direction.DESC, "id"));
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param name
	 * @return
	 */
	@ApiOperation(value = "查询考试批次(包含有效、无效)")
	@GetMapping("queryByName")
	public List<ExamEntity> queryByName(@RequestParam(required = false) String name) {

		if (StringUtils.isBlank(name)) {
			List<ExamEntity> list = Lists.newArrayList();
			return list;
		}

		Specification<ExamEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			predicates.add(cb.equal(root.get("rootOrgId"), getRootOrgId()));
			predicates.add(cb.like(root.get("name"), toSqlSearchPattern(name)));

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};
		Sort sort = new Sort(Direction.DESC, "updateTime");
		List<ExamEntity> page = examRepo.findAll(specification, sort);

		return page;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param rootOrgId
	 * @param examType
	 * @return
	 */
	@ApiOperation(value = "根据机构ID和考试类型查询所有有效考试批次", notes = "根据机构ID和考试类型查询所有有效考试批次")
	@GetMapping("rootOrgId/{rootOrgId}/{examType}")
	public List<ExamEntity> getEnableExamByRootOrgId(@PathVariable Long rootOrgId,
			@PathVariable String examType) {
		ExamEntity examCriteria = new ExamEntity();
		examCriteria.setRootOrgId(rootOrgId);
		if (StringUtils.isNotBlank(examType)) {
			examCriteria.setExamType(ExamType.valueOf(examType));
		}
		examCriteria.setEnable(true);

		ExampleMatcher exampleMatcher = ExampleMatcher.matching().withMatcher("name", contains());
		Example<ExamEntity> examExample = Example.of(examCriteria, exampleMatcher);
		return examRepo.findAll(examExample, new Sort(Direction.DESC, "id"));
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examCriteria
	 * @return
	 */
	@ApiOperation(value = "查询所有考试批次(包含有效无效)", notes = "不分页带查询")
	@GetMapping("all/both")
	public List<ExamEntity> getAllExam(@ModelAttribute ExamEntity examCriteria) {
		examCriteria.setRootOrgId(getRootOrgId());
		ExampleMatcher exampleMatcher = ExampleMatcher.matching().withMatcher("name", contains());
		Example<ExamEntity> examExample = Example.of(examCriteria, exampleMatcher);
		return examRepo.findAll(examExample, new Sort(Direction.DESC, "id"));
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param name
	 * @return
	 */
	@ApiOperation(value = "根据名称查询考试批次")
	@GetMapping("byName/{name}")
	public ExamEntity getExamByName(@PathVariable String name) {
		User accessUser = getAccessUser();
		ExamEntity one = examRepo.findByNameAndRootOrgId(name, accessUser.getRootOrgId());
		if (null == one) {
			throw new StatusException("E-001004", "考试不存在");
		}
		return one;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examId
	 * @return
	 */
	@ApiOperation(value = "按ID查询考试批次", notes = "ID查询")
	@GetMapping("{examId}")
	public ExamDomain getExamById(@PathVariable Long examId) {
		ExamEntity one = examRepo.findOne(examId);
		validateRootOrgIsolation(one.getRootOrgId());

		ExamDomain domain = new ExamDomain();
		domain.setBeginTime(one.getBeginTime());
		domain.setDuration(one.getDuration());
		domain.setEnable(one.getEnable());
		domain.setEndTime(one.getEndTime());
		domain.setExamTimes(one.getExamTimes());
		domain.setExamType(one.getExamType());
		domain.setId(one.getId());
		domain.setName(one.getName());
		domain.setRemark(one.getRemark());
		domain.setRootOrgId(one.getRootOrgId());
		domain.setStarted(one.getBeginTime().before(new Date()));
		return domain;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param domain
	 * @return
	 */
	@ApiOperation(value = "新增考试批次", notes = "新增")
	@PostMapping()
	public ExamEntity addExam(@RequestBody ExamDomain domain) {
		return saveExam(domain);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param domain
	 * @return
	 */
	@ApiOperation(value = "更新考试批次", notes = "更新")
	@PutMapping()
	public ExamEntity updateExam(@RequestBody ExamDomain domain) {
		return saveExam(domain);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param domain
	 * @return
	 */
	private ExamEntity saveExam(ExamDomain domain) {
		trim(domain, true);

		User accessUser = getAccessUser();

		ExamInfo examInfo = new ExamInfo();
		examInfo.setBeginTime(domain.getBeginTime());
		examInfo.setDuration(domain.getDuration());
		examInfo.setEnable(domain.getEnable());
		examInfo.setEndTime(domain.getEndTime());
		examInfo.setExamTimes(domain.getExamTimes());
		examInfo.setExamType(domain.getExamType());
		examInfo.setId(domain.getId());
		examInfo.setName(domain.getName());
		examInfo.setRemark(domain.getRemark());
		examInfo.setRootOrgId(accessUser.getRootOrgId());

		Map<String, String> properties = domain.getProperties();
		if (null == properties) {
			properties = Maps.newHashMap();
		}
		examInfo.setProperties(properties);

		ExamEntity saved = examService.saveExam(examInfo);

		return saved;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examId
	 * @return
	 */
	@ApiOperation(value = "查询考试批次所有属性")
	@GetMapping("allProperties/{examId}")
	public Map<String, String> getAllExamProperties(@PathVariable Long examId) {

		Map<String, String> map = Maps.newHashMap();
		List<ExamPropertyEntity> list = examPropertyRepo.findByexamId(examId);
		for (ExamPropertyEntity cur : list) {
			ExamProperty ep = ExamProperty.getByKeyId(cur.getKeyId());
			map.put(ep.name(), cur.getValue());
		}

		return map;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examId
	 * @return
	 */
	@ApiOperation(value = "考试批次所有属性")
	@GetMapping("property/{examId}/{key}")
	public String getExamProperty(@PathVariable Long examId, @PathVariable String key) {
		ExamProperty ep = ExamProperty.valueOf(key);
		ExamPropertyEntity one = examPropertyRepo.findByexamIdAndKeyId(examId, ep.getKeyId());
		if (null == one) {
			return null;
		}
		return one.getValue();
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param id
	 * @return
	 */
	@ApiOperation(value = "按ID删除考试批次", notes = "删除")
	@DeleteMapping("{id}")
	public List<Long> deleteExam(@PathVariable String id) {
		List<Long> examIds = Stream.of(id.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		examService.deleteExam(examIds);
		return examIds;
	}

	@ApiOperation(value = "启用考试", notes = "启用考试")
	@PutMapping("enable/{ids}")
	public void enableUser(@PathVariable String ids) {
		List<Long> examIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long examId : examIds) {
			ExamEntity exam = examRepo.findOne(examId);
			exam.setEnable(true);
			examRepo.save(exam);
		}
	}

	@ApiOperation(value = "禁用考试", notes = "禁用考试")
	@PutMapping("disable/{ids}")
	public void disableUser(@PathVariable String ids) {
		List<Long> examIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long examId : examIds) {
			ExamEntity exam = examRepo.findOne(examId);
			exam.setEnable(false);
			examRepo.save(exam);
		}
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examId
	 * @param curPage
	 * @param pageSize
	 * @return
	 */
	@ApiOperation(value = "查询课程组集合", notes = "")
	@GetMapping("getCourseGroupListByExamId/{examId}/{curPage}/{pageSize}")
	public List<CourseGroupDomain> getCourseGroupListByExamId(@PathVariable Long examId,
			@PathVariable Integer curPage, @PathVariable Integer pageSize) {

		Pageable pageable = new PageRequest(curPage - 1, pageSize, Sort.Direction.DESC,
				"updateTime");
		List<CourseGroupEntity> groupList = courseGroupRepo.findAllByExamId(examId, pageable);

		List<CourseGroupDomain> ret = Lists.newArrayList();

		for (CourseGroupEntity curCourseGroup : groupList) {
			CourseGroupDomain bean = new CourseGroupDomain();
			bean.setBeginTime(curCourseGroup.getBeginTime());
			bean.setCreationTime(curCourseGroup.getCreationTime());
			bean.setDescription(curCourseGroup.getDescription());
			bean.setEndTime(curCourseGroup.getEndTime());
			bean.setExamId(curCourseGroup.getExamId());
			bean.setId(curCourseGroup.getId());
			bean.setName(curCourseGroup.getName());
			bean.setUpdateTime(curCourseGroup.getUpdateTime());

			List<Long> courseIdList = Lists.newArrayList();
			bean.setCourseIdList(courseIdList);

			List<CourseGroupRelationEntity> relationList = courseGroupRelationRepo
					.findAllByGroupId(bean.getId());

			for (CourseGroupRelationEntity cur : relationList) {
				courseIdList.add(cur.getCourseId());
			}

			ret.add(bean);
		}

		return ret;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examId
	 * @param courseId
	 * @return
	 */
	@ApiOperation(value = "通过课程ID查询课程组集合", notes = "")
	@GetMapping("getCourseGroupListByExamIdAndCourseId/{examId}/{courseId}")
	public List<CourseGroupDomain> getCourseGroupListByExamIdAndCourseId(@PathVariable Long examId,
			@PathVariable Long courseId) {

		List<CourseGroupRelationEntity> relationList = courseGroupRelationRepo
				.findAllByCourseIdAndExamId(courseId, examId);

		List<Long> groupIdList = Lists.newArrayList();
		for (CourseGroupRelationEntity cur : relationList) {
			Long groupId = cur.getGroupId();
			groupIdList.add(groupId);
		}

		List<CourseGroupEntity> groupList = courseGroupRepo
				.findAllByIdInOrderByUpdateTimeDesc(groupIdList);

		List<CourseGroupDomain> ret = Lists.newArrayList();

		for (CourseGroupEntity curCourseGroup : groupList) {
			CourseGroupDomain bean = new CourseGroupDomain();
			bean.setBeginTime(curCourseGroup.getBeginTime());
			bean.setCreationTime(curCourseGroup.getCreationTime());
			bean.setDescription(curCourseGroup.getDescription());
			bean.setEndTime(curCourseGroup.getEndTime());
			bean.setExamId(curCourseGroup.getExamId());
			bean.setId(curCourseGroup.getId());
			bean.setName(curCourseGroup.getName());
			bean.setUpdateTime(curCourseGroup.getUpdateTime());

			List<Long> courseIdList = Lists.newArrayList();
			bean.setCourseIdList(courseIdList);

			List<CourseGroupRelationEntity> curRelationList = courseGroupRelationRepo
					.findAllByGroupId(bean.getId());

			for (CourseGroupRelationEntity cur : curRelationList) {
				courseIdList.add(cur.getCourseId());
			}
		}

		return ret;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param courseGroupBean
	 * @param request
	 */
	@ApiOperation(value = "更新课程组", notes = "")
	@PutMapping("courseGroup")
	public void updateCourseGroup(@RequestBody CourseGroupDomain courseGroupBean,
			HttpServletRequest request) {

		CourseGroupEntity courseGroup = new CourseGroupEntity();
		courseGroup.setBeginTime(courseGroupBean.getBeginTime());
		courseGroup.setCreationTime(courseGroupBean.getCreationTime());
		courseGroup.setDescription(courseGroupBean.getDescription());
		courseGroup.setEndTime(courseGroupBean.getEndTime());
		courseGroup.setExamId(courseGroupBean.getExamId());
		courseGroup.setId(courseGroupBean.getId());
		courseGroup.setName(courseGroupBean.getName());
		courseGroup.setUpdateTime(courseGroupBean.getUpdateTime());

		courseGroupRepo.save(courseGroup);
		List<Long> courseIdList = courseGroupBean.getCourseIdList();

		List<CourseGroupRelationEntity> relationList = Lists.newArrayList();

		courseGroupRelationRepo.deleteByGroupId(courseGroup.getId());

		for (Long cur : courseIdList) {
			CourseGroupRelationEntity relation = new CourseGroupRelationEntity();
			relation.setCourseId(cur);
			relation.setExamId(courseGroupBean.getExamId());
			relation.setGroupId(courseGroup.getId());
			relationList.add(relation);
		}
		courseGroupRelationRepo.save(relationList);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param id
	 */
	@ApiOperation(value = "删除课程组", notes = "")
	@DeleteMapping("courseGroup/{id}")
	public void deleteCourseGroup(@PathVariable Long id) {
		courseGroupRepo.delete(id);
		courseGroupRelationRepo.deleteByGroupId(id);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param curPage
	 * @param pageSize
	 * @param examOrgDomain
	 * @return
	 */
	@ApiOperation(value = "查询考试相关的学习中心设置", notes = "")
	@GetMapping("getExamOrgList/{curPage}/{pageSize}")
	public PageInfo<ExamOrgDomain> getExamOrgList(@PathVariable Integer curPage,
			@PathVariable Integer pageSize, @ModelAttribute ExamOrgDomain examOrgDomain) {

		Long examId = examOrgDomain.getExamId();
		if (null == examOrgDomain.getExamId()) {
			throw new StatusException("E-001210", "examId is null");
		}
		ExamEntity examEntity = examRepo.findOne(examId);
		if (null == examEntity) {
			throw new StatusException("E-001250", "examId is wrong");
		}
		validateRootOrgIsolation(examEntity.getRootOrgId());

		Specification<ExamOrgEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();

			predicates.add(cb.equal(root.get("examId"), examOrgDomain.getExamId()));

			if (null != examOrgDomain.getOrgId()) {
				predicates.add(cb.equal(root.get("orgId"), examOrgDomain.getOrgId()));
			}

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		Pageable pageable = new PageRequest(curPage, pageSize, Sort.Direction.DESC, "updateTime");
		Page<ExamOrgEntity> page = examOrgRepo.findAll(specification, pageable);

		Iterator<ExamOrgEntity> iterator = page.iterator();
		List<ExamOrgDomain> domainList = Lists.newArrayList();

		while (iterator.hasNext()) {
			ExamOrgEntity next = iterator.next();
			ExamOrgDomain bean = new ExamOrgDomain();
			domainList.add(bean);

			bean.setBeginTime(next.getBeginTime());
			bean.setEndTime(next.getEndTime());
			bean.setExamId(next.getExamId());
			bean.setId(next.getId());
			bean.setOrgId(next.getOrgId());
			bean.setRootOrgId(next.getRootOrgId());
			bean.setUpdateTime(next.getUpdateTime());

			GetOrgReq getOrgReq = new GetOrgReq();
			getOrgReq.setOrgId(bean.getOrgId());
			GetOrgResp getOrgResp = orgCloudService.getOrg(getOrgReq);
			OrgBean org = getOrgResp.getOrg();
			bean.setOrgName(org.getName());
			bean.setOrgCode(org.getCode());

			List<ExamOrgPropertyEntity> propList = examOrgPropertyRepo
					.findByexamIdAndOrgId(next.getExamId(), next.getOrgId());

			Map<String, String> map = Maps.newHashMap();
			for (ExamOrgPropertyEntity cur : propList) {
				ExamProperty ep = ExamProperty.getByKeyId(cur.getKeyId());
				map.put(ep.name(), cur.getValue());
			}

			bean.setProperties(map);
		}

		PageInfo<ExamOrgDomain> ret = new PageInfo<ExamOrgDomain>();
		ret.setList(domainList);
		ret.setTotal(page.getTotalElements());
		return ret;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examOrg
	 * @return
	 */
	@ApiOperation(value = "新增考试相关的学习中心设置", notes = "")
	@PostMapping("examOrg")
	public ExamOrgEntity addExamOrg(@RequestBody ExamOrgDomain domain) {
		return saveExamOrg(domain);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examOrg
	 * @return
	 */
	@ApiOperation(value = "更新考试相关的学习中心设置", notes = "")
	@PutMapping("examOrg")
	public ExamOrgEntity updateExamOrg(@RequestBody ExamOrgDomain domain) {
		return saveExamOrg(domain);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param domain
	 * @return
	 */
	private ExamOrgEntity saveExamOrg(ExamOrgDomain domain) {
		Long examId = domain.getExamId();

		ExamEntity examEntity = examRepo.findOne(examId);
		if (null == examEntity) {
			throw new StatusException("E-001250", "examId is wrong");
		}
		validateRootOrgIsolation(examEntity.getRootOrgId());

		Long orgId = domain.getOrgId();

		GetOrgReq getOrgReq = new GetOrgReq();
		getOrgReq.setOrgId(orgId);
		GetOrgResp getOrgResp = orgCloudService.getOrg(getOrgReq);
		OrgBean org = getOrgResp.getOrg();
		org.getRootId();
		validateRootOrgIsolation(org.getRootId());

		User accessUser = getAccessUser();
		ExamOrgInfo info = new ExamOrgInfo();
		info.setId(domain.getId());
		info.setBeginTime(domain.getBeginTime());
		info.setEndTime(domain.getEndTime());
		info.setExamId(domain.getExamId());
		info.setOrgId(domain.getOrgId());
		info.setRootOrgId(accessUser.getRootOrgId());

		Map<String, String> properties = domain.getProperties();
		if (null == properties) {
			properties = Maps.newHashMap();
		}
		info.setProperties(domain.getProperties());

		ExamOrgEntity ret = examService.saveExamOrg(info);
		return ret;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examId
	 * @param orgId
	 */
	@ApiOperation(value = "删除考试相关的学习中心设置", notes = "")
	@DeleteMapping("examOrg/{examId}/{orgId}")
	public void deleteExamOrg(@PathVariable Long examId, @PathVariable Long orgId) {
		examOrgRepo.deleteByExamIdAndOrgId(examId, orgId);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param request
	 * @param examId
	 * @return
	 */
	@ApiOperation(value = "考试IP限制", notes = "")
	@GetMapping("ipLimit/{examId}")
	public Map<String, Object> ipLimit(HttpServletRequest request, @PathVariable Long examId) {
		ExamEntity exam = examRepo.findOne(examId);
		if (null == exam) {
			throw new StatusException("E-001010", "考试不存在");
		}

		Map<String, Object> map = Maps.newHashMap();

		ExamPropertyEntity ipLimitProperty = examPropertyRepo.findByexamIdAndKeyId(exam.getId(),
				ExamProperty.IP_LIMIT.getKeyId());

		Boolean ipLimit = null;
		if (null != ipLimitProperty) {
			ipLimit = "true".equalsIgnoreCase(ipLimitProperty.getValue());
		}

		if (null == ipLimit || !ipLimit) {
			map.put("limited", false);
			map.put("desc", "未配置IP限制");
			return map;
		}

		String realIp = request.getHeader("x-forwarded-for");
		if (StringUtils.isBlank(realIp)) {
			realIp = request.getHeader("x-real-ip");
		}
		if (StringUtils.isBlank(realIp)) {
			map.put("limited", true);
			map.put("desc", "网络受限");
			return map;
		}
		realIp = realIp.trim();

		ExamPropertyEntity ipAddressesProperty = examPropertyRepo.findByexamIdAndKeyId(exam.getId(),
				ExamProperty.IP_ADDRESSES.getKeyId());

		String ipAddresses = null;
		if (null != ipAddressesProperty) {
			ipAddresses = ipAddressesProperty.getValue();
		}

		boolean limited = true;
		if (StringUtils.isNotBlank(ipAddresses)) {
			String[] arr = StringUtils.split(ipAddresses, ';');

			for (String cur : arr) {
				String ip = StringUtils.replace(cur.trim(), ".", "\\.");
				ip = StringUtils.replace(ip, "*", "\\w+");
				if (realIp.matches(ip)) {
					limited = false;
					map.put("desc", "IP段配置放行");
					break;
				}
			}
		}

		User accessUser = getAccessUser();
		Long orgId = accessUser.getOrgId();

		if (limited) {
			String key = "IP_" + orgId;
			String value = redisClient.get(key, String.class);
			if (null == value) {
				map.put("desc", "无机构管理员登录");
			} else {
				@SuppressWarnings("unchecked")
				Set<String> userKeyList = JsonUtil.fromJson(value, Set.class);

				for (String userKey : userKeyList) {
					User curUser = redisClient.get(userKey, User.class);
					if (null != curUser) {
						String clientIp = curUser.getClientIp();
						if (null != clientIp) {
							// IP取前三段
							clientIp = clientIp.substring(0, clientIp.lastIndexOf(".") + 1);
							if (realIp.startsWith(clientIp)) {
								limited = false;
								map.put("desc", "机构管理员[key=" + userKey + "]登录放行");
								break;
							}
						}
					}

				}
			}
		}

		map.put("limited", limited);

		return map;
	}

}
