package cn.com.qmth.examcloud.core.examwork.service.impl;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.stereotype.Service;

import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.api.OrgCloudService;
import cn.com.qmth.examcloud.core.examwork.base.enums.ExamProperty;
import cn.com.qmth.examcloud.core.examwork.dao.ExamOrgPropertyRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamOrgRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamPropertyRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamOrgEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamOrgPropertyEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamPropertyEntity;
import cn.com.qmth.examcloud.core.examwork.service.ExamService;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamInfo;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamOrgInfo;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年8月17日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Service
public class ExamServiceImpl implements ExamService {
	@Autowired
	ExamRepo examRepo;

	@Autowired
	ExamStudentServiceImpl examStudentService;

	@Autowired
	ExamOrgRepo examOrgRepo;

	@Autowired
	OrgCloudService orgCloudService;

	@Autowired
	ExamPropertyRepo examPropertyRepo;

	@Autowired
	ExamOrgPropertyRepo examOrgPropertyRepo;

	/**
	 * 删除考试批次
	 * 
	 * @param ids
	 * @return
	 */
	public void deleteExam(List<Long> ids) {
		List<ExamEntity> exams = examRepo.findByIdIn(ids);
		for (ExamEntity exam : exams) {
			if (exam.getBeginTime().before(new Date())) {
				throw new StatusException("", "[ " + exam.getName() + " ] 已经开考,不能删除");
			}
			// 删除学习中心考试时间
			examOrgRepo.deleteByExamId(exam.getId());
			examStudentService.deleteExamStudentsByExamId(exam.getId());
			examRepo.delete(exam);
		}
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examName
	 * @param rootOrgId
	 * @return
	 */
	public ExamEntity findExamByNameAndRootOrgId(String examName, Long rootOrgId) {
		ExamEntity exam = new ExamEntity();
		exam.setName(examName);
		exam.setRootOrgId(rootOrgId);
		return examRepo.findOne(Example.of(exam));
	}

	/*
	 * 实现
	 *
	 * @author WANGWEI
	 * 
	 * @see
	 * cn.com.qmth.examcloud.core.examwork.service.ExamService#saveExam(cn.com.
	 * qmth.examcloud.core.examwork.service.bean.ExamInfo)
	 */
	@Override
	public ExamEntity saveExam(ExamInfo examInfo) {

		ExamEntity exam = new ExamEntity();
		exam.setId(examInfo.getId());
		exam.setBeginTime(examInfo.getBeginTime());
		exam.setDuration(examInfo.getDuration());
		exam.setEnable(examInfo.getEnable());
		exam.setEndTime(examInfo.getEndTime());
		exam.setExamTimes(examInfo.getExamTimes());
		exam.setExamType(examInfo.getExamType());
		exam.setId(examInfo.getId());
		exam.setName(examInfo.getName());
		exam.setRemark(examInfo.getRemark());
		exam.setRootOrgId(examInfo.getRootOrgId());

		if (null == exam.getExamType()) {
			throw new StatusException("E-001005", "考试类型为空");
		}

		Map<String, String> properties = examInfo.getProperties();
		Map<ExamProperty, String> map = checkAndGetExamProperties(properties);

		ExamEntity findOne = examRepo.findByNameAndRootOrgId(exam.getName(), exam.getRootOrgId());
		if (null != findOne && !findOne.getId().equals(exam.getId())) {
			throw new StatusException("E-001001", "考试名称已存在");
		}

		ExamEntity saved = examRepo.save(exam);

		for (Entry<ExamProperty, String> entry : map.entrySet()) {
			ExamProperty prop = entry.getKey();
			String value = entry.getValue();
			ExamPropertyEntity entity = examPropertyRepo.findByexamIdAndKeyId(saved.getId(),
					prop.getKeyId());
			if (null == entity) {
				entity = new ExamPropertyEntity();
				entity.setExamId(saved.getId());
				entity.setKeyId(prop.getKeyId());
			}
			entity.setValue(value);

			examPropertyRepo.save(entity);
		}

		return saved;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param properties
	 * @return
	 */
	private Map<ExamProperty, String> checkAndGetExamProperties(Map<String, String> properties) {
		Map<ExamProperty, String> map = Maps.newHashMap();
		for (Entry<String, String> entry : properties.entrySet()) {
			String key = entry.getKey();
			String value = entry.getValue();
			if (StringUtils.isBlank(value)) {
				continue;
			}
			ExamProperty ep = null;
			try {
				ep = ExamProperty.valueOf(key);
			} catch (Exception e) {
				throw new StatusException("E-001004", "考试属性错误");
			}
			map.put(ep, value.trim());
		}

		String beforeExamRemark = map.get(ExamProperty.BEFORE_EXAM_REMARK);
		if (null != beforeExamRemark && beforeExamRemark.length() > 250000) {
			throw new StatusException("E-001002", "考前说明内容过大");
		}

		String afterExamRemark = map.get(ExamProperty.AFTER_EXAM_REMARK);
		if (null != afterExamRemark && afterExamRemark.length() > 250000) {
			throw new StatusException("E-001002", "考后说明内容过大");
		}

		String cheatingRemark = map.get(ExamProperty.CHEATING_REMARK);
		if (null != cheatingRemark && cheatingRemark.length() > 250000) {
			throw new StatusException("E-001002", "作弊说明内容过大");
		}

		return map;
	}

	/*
	 * 实现
	 *
	 * @author WANGWEI
	 * 
	 * @see
	 * cn.com.qmth.examcloud.core.examwork.service.ExamService#saveExamOrg(cn.
	 * com.qmth.examcloud.core.examwork.service.bean.ExamOrgInfo)
	 */
	@Override
	public ExamOrgEntity saveExamOrg(ExamOrgInfo examOrgInfo) {
		ExamOrgEntity examOrgEntity = null;

		if (null == examOrgInfo.getId()) {
			examOrgEntity = examOrgRepo.findByExamIdAndOrgId(examOrgInfo.getExamId(),
					examOrgInfo.getOrgId());
			if (null == examOrgEntity) {
				examOrgEntity = new ExamOrgEntity();
			}
		} else {
			examOrgEntity = examOrgRepo.findOne(examOrgInfo.getId());
			if (null == examOrgEntity) {
				throw new StatusException("E-001101", "id is wrong");
			}
		}

		examOrgEntity.setBeginTime(examOrgInfo.getBeginTime());
		examOrgEntity.setEndTime(examOrgInfo.getEndTime());
		examOrgEntity.setRootOrgId(examOrgInfo.getRootOrgId());
		examOrgEntity.setExamId(examOrgInfo.getExamId());
		examOrgEntity.setOrgId(examOrgInfo.getOrgId());

		Map<ExamProperty, String> map = checkAndGetExamProperties(examOrgInfo.getProperties());

		ExamOrgEntity saved = examOrgRepo.save(examOrgEntity);

		for (Entry<ExamProperty, String> entry : map.entrySet()) {
			ExamProperty prop = entry.getKey();
			String value = entry.getValue();
			ExamOrgPropertyEntity entity = examOrgPropertyRepo.findByexamIdAndOrgIdAndKeyId(
					saved.getExamId(), saved.getOrgId(), prop.getKeyId());
			if (null == entity) {
				entity = new ExamOrgPropertyEntity();
				entity.setExamId(saved.getExamId());
				entity.setOrgId(saved.getOrgId());
				entity.setKeyId(prop.getKeyId());
			}
			entity.setValue(value);

			examOrgPropertyRepo.save(entity);
		}

		return saved;
	}
}
